/*
* Program Name: files.c
* Author: Pat Y. Boens 
* Copyright (c) 1995 by The DVL Group
*Ŀ
* Created: 26 July 1995 at 10:27                                          
*                                                                         
* ....................................................................... 
* Revision number : 1.0 Last Revised : 26 July 1995 at 10:27              
* Description     : File Handling module.                                 
* ....................................................................... 
* Revision number : 3.3 Last Revised : 22 February 1996 at 9:08           
* Description     : GetTempPath()                                         
*                   FIL_CompareTime()                                     
*                   FIL_FileType()                                        
*                   FIL_IsDirectory() revisited                           
*                   FIL_IsArchived() revisited                            
*                   FIL_IsCompressed() revisited                          
*                   FIL_IsNormal() revisited                              
*                   FIL_IsHidden() revisited                              
*                   FIL_IsDirectory() revisited                           
*                   FIL_IsReadOnly() revisited                            
*                   FIL_IsTmp() revisited                                 
* ....................................................................... 
*                                                                         
* Part of FOCUS                                                           
*                                                                         
*                     Ŀ                   
*Ĵ The DVL Group rights reserved 
*                      
*/
#include <windows.h>
#include <string.h>
#include <stdlib.h>
#include <commdlg.h>
#include <stdio.h>
#include <lzexpand.h>
#include <direct.h>
#include <io.h>
#include <sys\types.h>
#include <sys\stat.h>
#include <shlobj.h>

#include "pro_ext.h"
#include "focus.h"
#include "files.h"

#define  PATH_LENGTH              256
#define  BUFFERSPACE              4096
#define  IS_WINDOWS_NT()          (! ( GetVersion() & 0x80000000 ))


//extern char _far *FOCUS_keep_path( char _far *cFile,char _far *pname );
//extern char _far *FOCUS_FIL_gettim( int Handle,char _far *Buffer );

struct _stat buf;
int    validStat  = 0;
long   hFindFirst = -1L;
struct _finddata_t c_file;

int ObtainShortFileName( LPCTSTR File,LPTSTR Result,DWORD Size )
/*------------------------------------------------------------*/
{
   return ( (int) GetShortPathName( File,Result,Size ) );
}

FOCUSFNC FW_FIL_fndexe( XBASE_PARAMETERS )
/*--------------------------------------*/
{
   char Path[PATH_LENGTH];

   _initparc(1); _initparc(2);

   FindExecutable( _parc(1),_parc(2),(LPSTR) Path );

   _retc( Path );
   _deinitparc(1);

   FOCUSFNCRETURN();
}

FOCUSFNC FW_FIL_info( XBASE_PARAMETERS )
/*------------------------------------*/
{
   _initparc(1);

  _retl( validStat = ! _stat( _parc(1),&buf ) );

   _deinitparc(1);
   FOCUSFNCRETURN();
}

FOCUSFNC FW_FIL_owner( XBASE_PARAMETERS )
/*-------------------------------------*/
{
   HANDLE hFile;
   SECURITY_ATTRIBUTES MySecurity;

   if ( ! IS_WINDOWS_NT() )
     {
        _initparc(1);

		MySecurity.bInheritHandle = TRUE;

        hFile = CreateFile( _parc(1)                           ,
                            GENERIC_READ                       ,
                            FILE_SHARE_READ | FILE_SHARE_WRITE ,
                            NULL ,//(LPSECURITY_ATTRIBUTES) &MySecurity,
                            OPEN_EXISTING                      ,
                            FILE_ATTRIBUTE_NORMAL              ,
                            NULL
                          );

        if ( hFile != INVALID_HANDLE_VALUE )
          {
             _retc( "OK" );
             CloseHandle( hFile );
          }
        else
          {
             _retc( "KO" );
          }

        _deinitparc(1);
     }
   else
     {
        _retc( "No NT" );
     }

   FOCUSFNCRETURN();
}

FOCUSFNC FW_FIL_isdir( XBASE_PARAMETERS )
/*-------------------------------------*/
{
   struct _stat FileInfo;

   if ( _stat( _parc(1),&FileInfo ) == 0 )
     _retl( ( ( FileInfo.st_mode & 64 ) >> 6 ));
   else
     _retl(0);

   FOCUSFNCRETURN();
}

FOCUSFNC FW_FIL_isize( XBASE_PARAMETERS )
/*-------------------------------------*/
{
   _retni( validStat ? buf.st_size : -1 );
   FOCUSFNCRETURN();
}

FOCUSFNC FW_FIL_idev( XBASE_PARAMETERS )
/*------------------------------------*/
{
   _retni( validStat ? buf.st_dev : -1 );
   FOCUSFNCRETURN();
}

FOCUSFNC FW_FIL_imtime( XBASE_PARAMETERS )
/*--------------------------------------*/
{
   _retni( validStat ? buf.st_mtime : -1 );
   FOCUSFNCRETURN();
}

FOCUSFNC FW_FIL_iatime( XBASE_PARAMETERS )
/*--------------------------------------*/
{
   _retni( validStat ? buf.st_atime : -1 );
   FOCUSFNCRETURN();
}

FOCUSFNC FW_FIL_ictime( XBASE_PARAMETERS )
/*--------------------------------------*/
{
   _retni( validStat ? buf.st_ctime : -1 );
   FOCUSFNCRETURN();
}

FOCUSFNC FW_FIL_exenam( XBASE_PARAMETERS )
/*--------------------------------------*/
{
   char ExeName[128];
   _StrCpy( ExeName,(char _far *) _pgmptr );
   _retc( ExeName );
   FOCUSFNCRETURN();
}

FOCUSFNC FW_FIL_path( XBASE_PARAMETERS )
/*------------------------------------*/
{
   char path_buffer[_MAX_PATH];
   char drive[_MAX_DRIVE];
   char dir[_MAX_DIR];
   char fname[_MAX_FNAME];
   char ext[_MAX_EXT];

   _initparc(1);

   strcpy( path_buffer,_parc(1) );

   _splitpath( path_buffer,drive,dir,fname,ext );

   _retc( dir );

   _deinitparc(1);

   FOCUSFNCRETURN();
}

FOCUSFNC FW_FIL_setatt( XBASE_PARAMETERS )
/*--------------------------------------*/
{
   _initparc(1);

   _retl( SetFileAttributes( _parc(1),_parni(2) ) );

   _deinitparc(1);

   FOCUSFNCRETURN();
}

FOCUSFNC FW_FIL_getatt( XBASE_PARAMETERS )
/*--------------------------------------*/
{
   _initparc(1);

   _retni( GetFileAttributes( _parc(1) ) );

   _deinitparc(1);

   FOCUSFNCRETURN();
}

int FW_IsFileOfThatType( char *cFile,int nType )
/*--------------------------------------------*/
{
   DWORD fAttrib;

   fAttrib = GetFileAttributes( cFile );

   if ( fAttrib != 0xFFFFFFFF )
      {
         switch ( nType )
            {
               case FW_READONLY  : return (   fAttrib & FILE_ATTRIBUTE_READONLY           ); break;
               case FW_HIDDEN    : return ( ( fAttrib & FILE_ATTRIBUTE_HIDDEN     ) >> 1  ); break;
               case FW_SYSTEM    : return ( ( fAttrib & FILE_ATTRIBUTE_SYSTEM     ) >> 2  ); break;
               case FW_DIRECTORY : return ( ( fAttrib & FILE_ATTRIBUTE_DIRECTORY  ) >> 4  ); break;
               case FW_ARCHIVED  : return ( ( fAttrib & FILE_ATTRIBUTE_ARCHIVE    ) >> 5  ); break;
               case FW_NORMALFILE: return ( ( fAttrib & FILE_ATTRIBUTE_NORMAL     ) >> 7  ); break;
               case FW_TMP       : return ( ( fAttrib & FILE_ATTRIBUTE_TEMPORARY  ) >> 8  ); break;
               case FW_COMPRESSED: return ( ( fAttrib & FILE_ATTRIBUTE_COMPRESSED ) >> 11 ); break;
               default           : return 0;
            }
	  }
   else
     {
        return 0;
     }
}

FOCUSFNC FW_FIL_isReadOnly( XBASE_PARAMETERS )
/*------------------------------------------*/
{
   _initparc(1);

   _retl( FW_IsFileOfThatType( _parc(1),FW_READONLY ) );

   _deinitparc(1);

   FOCUSFNCRETURN();
}

FOCUSFNC FW_FIL_isHidden( XBASE_PARAMETERS )
/*----------------------------------------*/
{
   _initparc(1);

   _retl( FW_IsFileOfThatType( _parc(1),FW_HIDDEN ) );

   _deinitparc(1);

   FOCUSFNCRETURN();
}

FOCUSFNC FW_FIL_isSystem( XBASE_PARAMETERS )
/*----------------------------------------*/
{
   _initparc(1);

   _retl( FW_IsFileOfThatType( _parc(1),FW_SYSTEM ) );

   _deinitparc(1);

   FOCUSFNCRETURN();
}

FOCUSFNC FW_FIL_isDirectory( XBASE_PARAMETERS )
/*-------------------------------------------*/
{
   _initparc(1);

   _retl( FW_IsFileOfThatType( _parc(1),FW_DIRECTORY ) );

   _deinitparc(1);

   FOCUSFNCRETURN();
}

FOCUSFNC FW_FIL_isArchived( XBASE_PARAMETERS )
/*-------------------------------------------*/
{
   _initparc(1);

   _retl( FW_IsFileOfThatType( _parc(1),FW_ARCHIVED ) );

   _deinitparc(1);

   FOCUSFNCRETURN();
}

FOCUSFNC FW_FIL_isNormal( XBASE_PARAMETERS )
/*-------------------------------------------*/
{
   _initparc(1);

   _retl( FW_IsFileOfThatType( _parc(1),FW_NORMALFILE ) );

   _deinitparc(1);

   FOCUSFNCRETURN();
}

FOCUSFNC FW_FIL_isTmp( XBASE_PARAMETERS )
/*-------------------------------------*/
{
   _initparc(1);

   _retl( FW_IsFileOfThatType( _parc(1),FW_TMP ) );

   _deinitparc(1);

   FOCUSFNCRETURN();
}

FOCUSFNC FW_FIL_isCompressed( XBASE_PARAMETERS )
/*--------------------------------------------*/
{
   _initparc(1);

   _retl( FW_IsFileOfThatType( _parc(1),FW_COMPRESSED ) );

   _deinitparc(1);

   FOCUSFNCRETURN();
}

FOCUSFNC FW_FIL_where( XBASE_PARAMETERS )
/*--------------------------------------*/
{
   char _far *cFile;
   char _far *cEnvironment;
   char cPath[128];

   _initparc(1); _initparc(2);

   cFile        = _parc(1);                                // Get file name to look for
   cEnvironment = _parc(2);                                // Get environment variable name

   _searchenv( cFile,cEnvironment,cPath );           // Search for file

   _retc( cPath );

   _deinitparc(1); _deinitparc(2);

   FOCUSFNCRETURN();
}

FOCUSFNC FW_FIL_mkdir( XBASE_PARAMETERS )
/*-------------------------------------*/
{
   _initparc(1);

   _retl( ! mkdir( _parc(1) ) );

   _deinitparc(1);

   FOCUSFNCRETURN();
}

FOCUSFNC FW_FIL_rmdir( XBASE_PARAMETERS )
/*-------------------------------------*/
{
   _initparc(1);

   _retl( ! rmdir( _parc(1) ) );

   _deinitparc(1);

   FOCUSFNCRETURN();
}

FOCUSFNC FW_FIL_chdir( XBASE_PARAMETERS )
/*-------------------------------------*/
{
   _initparc(1);

   _retl( ! chdir( _parc(1) ) );

   _deinitparc(1);

   FOCUSFNCRETURN();
}

//FOCUSFNC FW_handles_left()
///*---------------------*/
//{
//   register int i;
//   int handles[256], fcount;
//
//   for ( i = 1; i <= 256; i++ )
//      {
//         handles[i] = _FOpen( "NUL",0 );
//         if ( handles[i-1] < 0 )
//            { break; }
//      }
//
//   fcount = i - 1;
//
//   for ( i = 0; i <= fcount; i++)
//      {
//         _FClose( handles[i] );
//      }
//
//   return (fcount);
//}

//FOCUSFNC FW_FIL_hleft()
///*-------------------*/
//{
//   _retni( FW_handles_left() );
//   FOCUSFNCRETURN();
//}

FOCUSFNC FW_FIL_isbof( XBASE_PARAMETERS )
/*-------------------------------------*/
{
   long StartPosition;
   long OldPosition;
   int Handle;

   Handle        = _parni(1);                        // File handle

   OldPosition   = _FSeek( Handle,0,1 );             // Saving current position
   StartPosition = _FSeek( Handle,0,0 );             // Move to BOF

   _FSeek( Handle,OldPosition,0 );                   // Reset to old position

   _retl( OldPosition == StartPosition );            // Returns ISBOF
   FOCUSFNCRETURN();
}

FOCUSFNC FW_FIL_iseof( XBASE_PARAMETERS )
/*-------------------------------------*/
{
   long EndPosition;
   long OldPosition;
   int Handle;

   Handle        = _parni(1);                        // File handle

   OldPosition   = _FSeek( Handle,0,1 );             // Saving current position
   EndPosition   = _FSeek( Handle,0,2 );             // Move to EOF

   _FSeek( Handle,OldPosition,0 );                   // Reset to old position

   _retl( OldPosition == EndPosition );              // Returns ISEOF
   FOCUSFNCRETURN();
}

long FileLen( int Handle )
/*----------------------*/
{
   long StartPosition;
   long EndPosition;
   long OldPosition;

   OldPosition   = _FSeek( Handle,0,1 );             // Saving current position
   StartPosition = _FSeek( Handle,0,0 );             // Move to BOF
   EndPosition   = _FSeek( Handle,0,2 );             // Move to EOF

   _FSeek( Handle,OldPosition,0 );                   // Reset to old position

   return EndPosition - StartPosition;
}

FOCUSFNC FW_FIL_len( XBASE_PARAMETERS )
/*-----------------------------------*/
{
   _retnl( FileLen( _parni(1) ) );
   FOCUSFNCRETURN();
}

FOCUSFNC FW_FIL_tell( XBASE_PARAMETERS )
/*------------------------------------*/
{
   _retni( _FSeek( _parni(1),0,1 ) );          // Returns file pointer position
   FOCUSFNCRETURN();
}

FOCUSFNC FW_FIL_del( XBASE_PARAMETERS )
/*-----------------------------------*/
{
   _initparc(1);

   _retl( remove( _parc(1) ) == 0 );                     // return it !

   _deinitparc(1);

   FOCUSFNCRETURN();
}

FOCUSFNC FW_FIL_sethan( XBASE_PARAMETERS )
/*--------------------------------------*/
{
   _retni( SetHandleCount( _parni(1) ) );
   FOCUSFNCRETURN();
}

FOCUSFNC FW_FIL_wriini( XBASE_PARAMETERS )
/*--------------------------------------*/
{
   char _far *Section,*Entry,*String,*INIFile;

   _initparc(1); _initparc(2); _initparc(3); _initparc(4);

   Section   = _parc(1);
   Entry     = _parc(2);
   String    = _parc(3);
   INIFile   = _parc(4);

   _retl( WritePrivateProfileString( Section,Entry,String,INIFile ) );

   _deinitparc(1); _deinitparc(2); _deinitparc(3); _deinitparc(4);

   FOCUSFNCRETURN();
}

FOCUSFNC FW_FIL_reaini( XBASE_PARAMETERS )
/*--------------------------------------*/
{
   char _far *Section,*Entry,*INIFile;
   char szBuf[133];

   _initparc(1); _initparc(2); _initparc(3);

   Section = _parc(1);                      // Handle to pointer : 1st param
   Entry   = _parc(2);                      // Handle to pointer : 2nd param
   INIFile = _parc(3);                      // Handle to pointer : 3rd param

   GetPrivateProfileString( Section,Entry,"",szBuf,sizeof(szBuf),INIFile );

   _retc( szBuf );

   _deinitparc(1); _deinitparc(2); _deinitparc(3);

   FOCUSFNCRETURN();
}

int FW_WriteWinIni( char *section,char *entry,char *value )
/*-------------------------------------------------------*/
{
   return ( WriteProfileString( section,entry,value ) );
}

FOCUSFNC FW_FIL_wwriin( XBASE_PARAMETERS )
/*--------------------------------------*/
{
   _initparc(1); _initparc(2); _initparc(3);

   _retl( WriteProfileString( _parc(1),_parc(2),_parc(3) ) );

   _deinitparc(1); _deinitparc(2); _deinitparc(3);

   FOCUSFNCRETURN();
}

FOCUSFNC FW_FIL_wreain( XBASE_PARAMETERS )
/*--------------------------------------*/
{
   char _far *Section,*Entry;
   char szBuf[133];

   _initparc(1); _initparc(2);

   Section = _parc(1);                      // Handle to pointer : 1st param
   Entry   = _parc(2);                      // Handle to pointer : 2nd param

   GetProfileString( Section,Entry,"",szBuf,sizeof(szBuf) );

   _retc( szBuf );

   _deinitparc(1); _deinitparc(2);

   FOCUSFNCRETURN();
}

FOCUSFNC FW_FIL_wsecti( XBASE_PARAMETERS )
/*--------------------------------------*/
{
   PSTR pszBuf;
   char car,PrevChar = 'A';
   int i;

   _initparc(1);

   pszBuf = (PSTR) LocalAlloc( LMEM_FIXED,2048 );     // Allocate a buffer for the entries.

   GetProfileString( _parc(1),NULL,"",pszBuf,2048 );  // Retrieve all the entries for given section.

   for ( i=0;i<2048;i++ )
     {
        car = pszBuf[i];

        if ( car == 0 )
          {
             if ( PrevChar == 0 )
               {
                  pszBuf[i-1] = 0;
                  break;
               }
             else
               pszBuf[i] = ';';
          }

        PrevChar = car;
     }

   _retc( pszBuf );

   _deinitparc(1);
   LocalFree( (HANDLE) pszBuf );

   FOCUSFNCRETURN();
}

FOCUSFNC FW_FIL_sectio( XBASE_PARAMETERS )
/*---------------------------------------*/
{
   PSTR pszBuf;
   char car,PrevChar = 'A';
   int i;

   _initparc(1); _initparc(2);

   pszBuf = (PSTR) LocalAlloc( LMEM_FIXED,2048 );     // Allocate a buffer for the entries.

   GetPrivateProfileString( _parc(1),NULL,"",pszBuf,2048,_parc(2) ); // Retrieve all the entries for given section.

   for ( i=0;i<2048;i++ )
     {
        car = pszBuf[i];

        if ( car == 0 )
          {
             if ( PrevChar == 0 )
               {
                  pszBuf[i-1] = 0;
                  break;
               }
             else
               pszBuf[i] = ';';
          }

        PrevChar = car;
     }

   _retc( pszBuf );

   _deinitparc(1); _deinitparc(2);
   LocalFree( (HANDLE) pszBuf );

   FOCUSFNCRETURN();
}

FOCUSFNC FW_FIL_rendir( XBASE_PARAMETERS )
/*--------------------------------------*/
{

   _initparc(1); _initparc(2);

   _retl( rename( _parc(1),_parc(2) ) == 0 );         // Actual return to FoxPro

   _deinitparc(1); _deinitparc(2);

   FOCUSFNCRETURN();
}

FOCUSFNC FW_FIL_drive( XBASE_PARAMETERS )
/*-------------------------------------*/
{
   char path_buffer[_MAX_PATH];
   char drive[_MAX_DRIVE];
   char dir[_MAX_DIR];
   char fname[_MAX_FNAME];
   char ext[_MAX_EXT];

   _initparc(1);

   strcpy( path_buffer,_parc(1) );

   _splitpath( path_buffer,drive,dir,fname,ext );

   _retc( drive );

   FOCUSFNCRETURN();
}

char FAR *FW_FIL_fullname( char FAR *cFile,char FAR *cResult )
/*----------------------------------------------------------*/
{
   char FAR *p, FAR *q;

   _MemMove( cResult,cFile,_StrLen(cFile) + 1 );

   for ( p=cResult + _StrLen(cResult) - 1; p>=cResult && isspace(*p) ;p-- )
      { ; }

   *(++p) = 0;

   // Find the rightmost backslash and take everything after it.
   q = cResult;
   p = strrchr(q,'\\');

   if(p != NULL)
      q = p+1;

   // Find the rightmost remaining colon and take everything after it.
   p = strrchr(q,':');

   if(p != NULL)
      q = p+1;

   _MemMove( cResult,q,_StrLen(q)+1 );
   return( cResult);
}

FOCUSFNC FW_FIL_fname( XBASE_PARAMETERS )
/*-------------------------------------*/
{
   char temp[PATH_LENGTH];
   MHANDLE t;

   t = GET_STRING_HANDLE( 0 );

   LOCK_HANDLE(t);

   FW_FIL_fullname( _HandToPtr( t ),temp );
   _retc( temp );

   UNLOCK_HANDLE( t );
   FOCUSFNCRETURN();
}

FOCUSFNC FW_FIL_ext( XBASE_PARAMETERS )
/*-----------------------------------*/
{
   char path_buffer[_MAX_PATH];
   char drive[_MAX_DRIVE];
   char dir[_MAX_DIR];
   char fname[_MAX_FNAME];
   char ext[_MAX_EXT];

   _initparc(1);

   strcpy( path_buffer,_parc(1) );

   _splitpath( path_buffer,drive,dir,fname,ext );

   _retc( ext ); 

   _deinitparc(1);

   FOCUSFNCRETURN();
}

FOCUSFNC FW_FIL_name( XBASE_PARAMETERS )
/*------------------------------------*/
{
   char path_buffer[_MAX_PATH];
   char drive[_MAX_DRIVE];
   char dir[_MAX_DIR];
   char fname[_MAX_FNAME];
   char ext[_MAX_EXT];

   _initparc(1);

   strcpy( path_buffer,_parc(1) );

   _splitpath( path_buffer,drive,dir,fname,ext );

   _retc( fname );

   _deinitparc(1);

   FOCUSFNCRETURN();
}

FOCUSFNC FW_FIL_expand( XBASE_PARAMETERS )
/*--------------------------------------*/
{
   char FAR *szSrc;
   char FAR *szDst;

   OFSTRUCT ofStrSrc;
   OFSTRUCT ofStrDest;

   HFILE hfSrcFile,hfDstFile;

   _initparc(1); _initparc(2);

   szSrc     = _parc(1);                                // Source file
   szDst     = _parc(2);                                // Target file

   hfSrcFile = LZOpenFile( szSrc,&ofStrSrc ,OF_READ ); // Open the compressed source file.
   hfDstFile = LZOpenFile( szDst,&ofStrDest,OF_CREATE);

   LZCopy( hfSrcFile,hfDstFile );                      // SDK service

   LZClose( hfSrcFile );                               // Close source file
   LZClose( hfDstFile );                               // Close target file

   _deinitparc(1); _deinitparc(2);

   FOCUSFNCRETURN();
}

FOCUSFNC FW_FIL_copyTimes( XBASE_PARAMETERS )
/*-----------------------------------------*/
{
   char *Input;
   int  HandleIn = -1;

   _initparc(1);

   Input  = _parc(1);

   _try
      {
         if ( IsFile( Input ) )
            {
               if ( ( HandleIn = _FOpen( Input,FO_READONLY ) ) == -1 )
                 {
                    _retni( -2 );
                 }
               else
                 {
                    _retni( (int) ( FileLen( HandleIn ) / BUFFERSPACE ) + 1 );
                 }
            }
         else
            {
               _retni( -3 );
            }
      }				 // End of the try block

   _finally
      {
         if ( HandleIn != -1 )
            _FClose( HandleIn );

         _deinitparc(1);
      }

   FOCUSFNCRETURN();
}

FOCUSFNC FW_FIL_copy( XBASE_PARAMETERS )
/*------------------------------------*/
{
   char *Input;
   char *Output;
   char *Udf;
   char Buffer[BUFFERSPACE];
   char cSize1[10];
   char cSize2[10];
   char Command[60];

   int  nRetVal;
   int  HandleIn  = -1;
   int  HandleOut = -1;
   long BytesRead;
   int  Times;
   int  Count;

   memset( cSize1 ,0,sizeof( cSize1  ) );
   memset( cSize2 ,0,sizeof( cSize2  ) );
   memset( Command,0,sizeof( Command ) );

   _initparc(1); _initparc(2); _initparc(3) ;

   Input  = _parc(1);
   Output = _parc(2);
   Udf    = _parc(3);

   _try {
   if ( IsFile( Input ) )
      {
         //cTimeStamp = FIL_gtime( Input )           // Get time stamp
         //dDateStamp = FIL_gdate( Input )           // Get date stamp

        if ( PCOUNT() > 3 )
           {
              nRetVal = CopyFile( (LPCTSTR) Input,(LPCTSTR) Output,(BOOL) 1 );
              FOCUSFNCRETURN();
           }

         if ( ( HandleIn = _FOpen( Input,FO_READONLY ) ) == -1 )
           {
              nRetVal = -2;
           }
         else
           {
              if ( ( HandleOut = _FCreate( Output,FC_NORMAL ) ) == -1 )
                {
                   nRetVal = -1;
                }
              else
                {
                   Times = (int) ( FileLen( HandleIn ) / BUFFERSPACE ) + 1;
                   Count = 1;

                   itoa( Times,cSize1,10 );
                   itoa( Count,cSize2,10 );
                   strcat( Command,Udf );
                   strcat( Command,"(" );
                   strcat( Command,cSize1 );
                   strcat( Command,"," );
                   strcat( Command,cSize2 );
                   strcat( Command,")" );

                   _Execute( Command );

                   BytesRead = _FRead( HandleIn,Buffer,BUFFERSPACE );
                   nRetVal   = 0;

                   while ( BytesRead == BUFFERSPACE )
                     {
                       _FWrite( HandleOut,Buffer,BUFFERSPACE );
                       BytesRead = _FRead( HandleIn,Buffer,BUFFERSPACE );

                       ++Count;

                       memset( Command,0,sizeof( Command ) );
                       itoa( Times,cSize1,10 );
                       itoa( Count,cSize2,10 );
                       strcat( Command,Udf );
                       strcat( Command,"(" );
                       strcat( Command,cSize1 );
                       strcat( Command,"," );
                       strcat( Command,cSize2 );
                       strcat( Command,")" );
                       _Execute( Command );
                     }

                   _FWrite( HandleOut, Buffer, BytesRead );
                   //FIL_stime( Output,cTimeStamp )          // Reset time stamp
                   //FIL_sdate( Output,dDateStamp )          // Reset date stamp
                }
           }
	 	 }
   else
      {
         nRetVal = -3;
      }
   }				 // End of the try block

   _finally
      {
         if ( HandleOut != -1 )
            _FClose( HandleOut );

         if ( HandleIn != -1 )
            _FClose( HandleIn );

         _deinitparc(1); _deinitparc(2); _deinitparc(3) ;
         _retni( nRetVal );
      }

   FOCUSFNCRETURN();

}

FOCUSFNC FW_FIL_open( XBASE_PARAMETERS )
/*------------------------------------*/
{
    _initparc(1);
	_retni( _FOpen( _parc(1),_parni(2) ) );
	_deinitparc(1);
	FOCUSFNCRETURN();
}

FOCUSFNC FW_FIL_close( XBASE_PARAMETERS )
/*------------------------------------*/
{
    _retl( _FClose( _parni(1) ) == 0 );
	FOCUSFNCRETURN();
}

FOCUSFNC FW_FIL_first( XBASE_PARAMETERS )
/*-------------------------------------*/
{
   char *cFileSpec;
   
   _initparc(1);

   cFileSpec = _parc(1);

   if ( ( hFindFirst = _findfirst( cFileSpec,&c_file ) ) == -1L )
	   {
          _retc( "" );
	   }
	else
       {
	      _retc( c_file.name );
	   }

   _deinitparc(1);

   FOCUSFNCRETURN();
}

FOCUSFNC FW_FIL_next( XBASE_PARAMETERS )
/*-------------------------------------*/
{
   if ( hFindFirst != -1L )
	  {
       if ( _findnext( hFindFirst,&c_file ) == 0 )
         {
           _retc( c_file.name );
         }
		 else
         {
           _retc( "" ); _findclose( hFindFirst );
         }
     }
   else
     {
       _retc( "" );
     }

   FOCUSFNCRETURN();
}

FOCUSFNC FW_FIL_CompareTime( XBASE_PARAMETERS )
/*-------------------------------------------*/
{
   FILETIME Creation1;
   FILETIME LastAccess1;
   FILETIME LastWrite1;

   FILETIME Creation2;
   FILETIME LastAccess2;
   FILETIME LastWrite2;

   HANDLE hFile1,hFile2;

   char *cFile1,*cFile2;
   int nStructure;
   LONG nResult;

   _initparc(1);_initparc(2);

   cFile1 = _parc(1); cFile2 = _parc(2); nStructure = _parni(3);

   hFile1 = CreateFile( cFile1                            ,
                        GENERIC_READ                      ,
                        FILE_SHARE_READ | FILE_SHARE_WRITE,
                        NULL                              ,
                        OPEN_EXISTING                     ,
                        FILE_ATTRIBUTE_NORMAL             ,
                        NULL
                      );

   hFile2 = CreateFile( cFile2                            ,
                        GENERIC_READ                      ,
                        FILE_SHARE_READ | FILE_SHARE_WRITE,
                        NULL                              ,
                        OPEN_EXISTING                     ,
                        FILE_ATTRIBUTE_NORMAL             ,
                        NULL
                      );

   if ( hFile1 != INVALID_HANDLE_VALUE )
      {
         GetFileTime( (HANDLE) hFile1            ,
                      ( LPFILETIME ) &Creation1  ,
                      ( LPFILETIME ) &LastAccess1,
                      ( LPFILETIME ) &LastWrite1
                    );

		 CloseHandle( hFile1 );
	  }

   if ( hFile2 != INVALID_HANDLE_VALUE )
      {
         GetFileTime( (HANDLE) hFile2            ,
                      ( LPFILETIME ) &Creation2  ,
                      ( LPFILETIME ) &LastAccess2,
                      ( LPFILETIME ) &LastWrite2
                    );

		 CloseHandle( hFile2 );
	  }

   switch ( nStructure )
      {
         case 1 : nResult = CompareFileTime( (LPFILETIME) &LastWrite1   ,(LPFILETIME) &LastWrite2  ); break;
         case 2 : nResult = CompareFileTime( (LPFILETIME) &LastAccess1  ,(LPFILETIME) &LastAccess2 ); break;
         case 3 : ;
         default: nResult = CompareFileTime( (LPFILETIME) &Creation1    ,(LPFILETIME) &Creation2 );
      }

   _deinitparc(1); _deinitparc(2);

   _retni( (int) nResult );

   FOCUSFNCRETURN();
}

int FW_GetFileTimeDate( char *cFile,int nStructure,char *Result,int nType )
/*-----------------------------------------------------------------------*/
{
   FILETIME Creation;
   FILETIME LastAccess;
   FILETIME LastWrite;
   FILETIME LocalTime;

   LPFILETIME AppropriateStructure;

   WORD DOSTime;
   WORD DOSDate;

   HANDLE hFile;

   char itoaBuffer[10];

   int nSeconds;
   int nMinutes;
   int nHours;

   int nDay;
   int nMonth;
   int nYear;

   int nRetVal = 1;

   hFile = CreateFile( cFile                             ,
                       GENERIC_READ                      ,
                       FILE_SHARE_READ | FILE_SHARE_WRITE,
                       NULL                              ,
                       OPEN_EXISTING                     ,
                       FILE_ATTRIBUTE_NORMAL             ,
                       NULL
                     );

   if ( hFile != INVALID_HANDLE_VALUE )
      {
         if ( GetFileTime( (HANDLE) hFile            ,
                           ( LPFILETIME ) &Creation  ,
                           ( LPFILETIME ) &LastAccess,
                           ( LPFILETIME ) &LastWrite
                         )
            )
            {
               switch ( nStructure )
                  {
                     case 1 : AppropriateStructure = ( LPFILETIME ) &LastWrite;  break;
                     case 2 : AppropriateStructure = ( LPFILETIME ) &LastAccess; break;
                     case 3 : ;
                     default: AppropriateStructure = ( LPFILETIME ) &Creation;
                  }

               FileTimeToLocalFileTime( AppropriateStructure        ,
                                        ( LPFILETIME ) &LocalTime
                                      );

               if ( FileTimeToDosDateTime( ( LPFILETIME ) &LocalTime,
                                           ( LPWORD ) &DOSDate      ,
                                           ( LPWORD ) &DOSTime
                                         )
                  )
                  {

                     memset( itoaBuffer,0,sizeof( itoaBuffer ) );
                     memset( Result    ,0,sizeof( Result     ) );

                     if ( nType == FOCUS_NEEDS_FILETIME )
                        {
                           nHours   = (int) ( (DWORD)( DOSTime & HRMASK  ) >> HRSHIFT  );
                           nMinutes = (int) (        ( DOSTime & MINMASK ) >> MINSHIFT );
                           nSeconds = (int) (        ( DOSTime & SECMASK ) >> SECSHIFT ) * 2;

                           itoa( nHours,itoaBuffer,10 );

                           if ( nHours < 10 )
                              strcat( Result,"0" );

                           strcat( Result,itoaBuffer );
                           strcat( Result,":" );

                           if ( nMinutes < 10 )
                              strcat( Result,"0" );

                           itoa( nMinutes,itoaBuffer,10 );

                           strcat( Result,itoaBuffer );
                           strcat( Result,":" );

                           if ( nSeconds < 10 )
                              strcat( Result,"0" );

                           itoa( nSeconds,itoaBuffer,10 );

                           strcat( Result,itoaBuffer );
                        }
                     else                        // Date needed
                        {
                           nYear  = (int) ( ( ( DOSDate & YRMASK  ) >> YRSHIFT  ) + 1980 );
                           nMonth = (int)   ( ( DOSDate & MONMASK ) >> MONSHIFT );
                           nDay   = (int)   ( ( DOSDate & DAYMASK ) >> DAYSHIFT );

                           itoa( nYear,itoaBuffer,10 );
                           strcat( Result,itoaBuffer );

                           itoa( nMonth,itoaBuffer,10 );

                           if ( nMonth < 10 )
                              strcat( Result,"0" );

                           strcat( Result,itoaBuffer );

                           itoa( nDay,itoaBuffer,10 );

                           if ( nDay < 10 )
                              strcat( Result,"0" );

                           strcat( Result,itoaBuffer );
                        }
                  }
               else
                  {
                     nRetVal = 0;
                  }
            }
         else
            {
               nRetVal = 0;
            }

         CloseHandle( hFile );
      }
   else
      {
         nRetVal = 0;
      }

   return nRetVal;

}

FOCUSFNC FW_FIL_gtime( XBASE_PARAMETERS )
/*-------------------------------------*/
{
   char *cFile;
   char Result[9];
   int nStructure;

   _initparc(1);

   cFile = _parc(1);

	if ( PCOUNT() > 1 )
	   nStructure = _parni(2);
	else
	   nStructure = 1;

   if ( FW_GetFileTimeDate( cFile,nStructure,Result,FOCUS_NEEDS_FILETIME ) )
      _retc( Result );
   else
      _retc( "" );

   _deinitparc(1);

   FOCUSFNCRETURN();
}

FOCUSFNC FW_FIL_gdate( XBASE_PARAMETERS )
/*-------------------------------------*/
{
   char *cFile;
   char Result[9];
   int nStructure;

   _initparc(1);

   cFile = _parc(1);

	if ( PCOUNT() > 1 )
	   nStructure = _parni(2);
	else
	   nStructure = 1;

   if ( FW_GetFileTimeDate( cFile,nStructure,Result,FOCUS_NEEDS_FILEDATE ) )
      _retc( Result );
   else
      _retc( "" );

   _deinitparc(1);

   FOCUSFNCRETURN();
}

int FW_SetFileTimeDate( char *cFile,int nStructure,char *Buffer,int nType )
/*-----------------------------------------------------------------------*/
{
   FILETIME Creation;
   FILETIME LastAccess;
   FILETIME LastWrite;
   FILETIME LocalTime;

   LPFILETIME AppropriateStructure;

   WORD DOSTime;
   WORD DOSDate;

   char YY[5];
   char MM[3];
   char DD[3];

   char SS[3];
   char Mn[3];
   char HH[3];

   BOOL lSuccess;

   int nRetVal = 1;

   HANDLE hFile;

   hFile = CreateFile( cFile                             ,
                       GENERIC_READ | GENERIC_WRITE      ,
                       FILE_SHARE_READ | FILE_SHARE_WRITE,
                       NULL                              ,
                       OPEN_EXISTING                     ,
                       FILE_ATTRIBUTE_NORMAL             ,
                       NULL);

   if ( hFile != INVALID_HANDLE_VALUE )
      {
         // Obtain time and date info for all three structures
         if ( GetFileTime( (HANDLE) hFile            ,
                           ( LPFILETIME ) &Creation  ,
                           ( LPFILETIME ) &LastAccess,
                           ( LPFILETIME ) &LastWrite
                         )
            )
            {
               // What is the structure we need to update ?
               switch ( nStructure )
                  {
                     case 1 : AppropriateStructure = ( LPFILETIME ) &LastWrite;  break;
                     case 2 : AppropriateStructure = ( LPFILETIME ) &LastAccess; break;
                     case 3 : ;
                     default: AppropriateStructure = ( LPFILETIME ) &Creation;
                  }

               // Get Date and Time information into DOSTime and DOSDate for the appropriate structure
               if ( FileTimeToDosDateTime( AppropriateStructure     ,
                                           ( LPWORD ) &DOSDate      ,
                                           ( LPWORD ) &DOSTime
                                         )
                  )
                  {
                     if ( nType == FOCUS_NEEDS_FILETIME )
                        {
                           memset( HH,0,sizeof( HH ) );
                           memset( Mn,0,sizeof( Mn ) );
                           memset( SS,0,sizeof( SS ) );

                           strncpy( HH,Buffer    ,2 );
                           strncpy( Mn,Buffer + 3,2 );
                           strncpy( SS,Buffer + 6,2 );

                           DOSTime = ( WORD ) ( ( atoi( SS ) / 2 )    << SECSHIFT |
                                                  atoi( Mn )          << MINSHIFT |
                                                  atoi( HH )          << HRSHIFT );

                           // Assign the new date in the structure
                           DosDateTimeToFileTime( DOSDate,DOSTime,( LPFILETIME) &LocalTime );

                           LocalFileTimeToFileTime( ( LPFILETIME ) &LocalTime,
                                                    AppropriateStructure );

                        }
                     else                 // FOCUS_NEEDS_FILEDATE
                        {
                           // Setup temporary buffers to NULLs
                           memset( YY,0,sizeof( YY ) );
                           memset( MM,0,sizeof( MM ) );
                           memset( DD,0,sizeof( DD ) );

                           // Copy values into the temorary buffers
                           strncpy( YY,Buffer    ,4 );
                           strncpy( MM,Buffer + 4,2 );
                           strncpy( DD,Buffer + 6,2 );

                           // ... Date if we want to update the Date member
                           // ... Time if we want to update the time member
                           DOSDate = ( WORD ) ( ( atoi( YY ) - 1980 ) << YRSHIFT  |
                                                  atoi( MM )          << MONSHIFT |
                                                  atoi( DD )          << DAYSHIFT );

                           // Assign the new date in the structure
                           DosDateTimeToFileTime( DOSDate,DOSTime,AppropriateStructure );

                        }

                     // Now, it's time to update the file, based on a particular structure
                     switch ( nStructure )
                        {
                           // LastWrite
                           case 1 : lSuccess = SetFileTime( hFile,NULL,NULL,AppropriateStructure );
                                    break;
                           // LastAccess
                           case 2 : lSuccess = SetFileTime( hFile,NULL,AppropriateStructure,NULL );
                                    break;
                           // Creation
                           case 3 : ;
                           default: lSuccess = SetFileTime( hFile,AppropriateStructure,NULL,NULL );
                                    break;
                        }

                     if ( ! lSuccess )
                        nRetVal = 0;

                  }
               else                    // Couldn't obtain date and time info
                  {
                     nRetVal = 0;
                  }
            }
         else
            {
               nRetVal = 0;
            }

         CloseHandle( hFile );
      }
   else
      {
         nRetVal = 0;
      }

   return nRetVal;
}

FOCUSFNC FW_FIL_stime( XBASE_PARAMETERS )
/*-------------------------------------*/
{
   int nStructure;                     // LastWrite, LastAccess or Creation

   _initparc(1);                       // File name
   _initparc(2);                       // Time stamp

	if ( PCOUNT() > 2 )
	   nStructure = _parni(3);
	else
	   nStructure = 1;

   _retl( FW_SetFileTimeDate( _parc(1),nStructure,_parc(2),FOCUS_NEEDS_FILETIME ) );

   _deinitparc(1);
   _deinitparc(2);
   FOCUSFNCRETURN();
}

FOCUSFNC FW_FIL_sdate( XBASE_PARAMETERS )
/*-------------------------------------*/
{
   int nStructure;                     // LastWrite, LastAccess or Creation

   _initparc(1);                       // File name
   _initparc(2);                       // Date stamp

	if ( PCOUNT() > 2 )
	   nStructure = _parni(3);
	else
	   nStructure = 1;

   _retl( FW_SetFileTimeDate( _parc(1),nStructure,_parc(2),FOCUS_NEEDS_FILEDATE ) );

   _deinitparc(1);
   _deinitparc(2);
   FOCUSFNCRETURN();
}

FOCUSFNC FW_FIL_ShortName( XBASE_PARAMETERS )
/*-----------------------------------------*/
{
   char szShortPath[1025];

   _initparc(1);

   ObtainShortFileName( (LPCTSTR) _parc(1),(LPTSTR) szShortPath,(DWORD) 1024 );

   _retc( szShortPath );

   _deinitparc(1);

   FOCUSFNCRETURN();
}

FOCUSFNC FW_FIL_LongName( XBASE_PARAMETERS )
/*----------------------------------------*/
{
   char    szLongPath[1025];
   LPTSTR  szFilePart;

   _initparc(1);

   GetFullPathName( (LPCTSTR) _parc(1)  ,
                    1024                ,
                    (LPTSTR)  szLongPath,
                    &szFilePart
                   );

   _retc( szLongPath );

   _deinitparc(1);

   FOCUSFNCRETURN();
}

FOCUSFNC FW_FIL_split( XBASE_PARAMETERS )
/*-------------------------------------*/
{
   char path_buffer[_MAX_PATH];
   char drive[_MAX_DRIVE];
   char dir[_MAX_DIR];
   char fname[_MAX_FNAME];
   char ext[_MAX_EXT];

   _initparc(1);

   strcpy( path_buffer,_parc(1) );

   _splitpath( path_buffer,drive,dir,fname,ext );

   switch ( _parni(2) )
      {
	     case 1:  _retc( drive ); break;
		 case 2:  _retc( dir );	  break;
		 case 3:  _retc( fname ); break;
		 case 4:  ;
		 default: _retc( ext ); 
	  }

   _deinitparc(1);

   FOCUSFNCRETURN();
}

FOCUSFNC FW_FIL_GetFolderLocation( XBASE_PARAMETERS )
/*-------------------------------------------------*/
{
   LPITEMIDLIST pidlFolder;
   char pszPath[_MAX_PATH];

   pszPath[0] = '\0';

   //memset( pidlFolder,0,sizeof( LPITEMIDLIST ) );

   /*
   CSIDL_BITBUCKET         Recycle bin -- file system directory containing
                           file objects in the user's recycle bin. The
                           location of this directory is not in the registry;
                           it is marked with the hidden and system attributes
                           to prevent the user from moving or deleting it.
   CSIDL_CONTROLS          Control Panel -- virtual folder containing icons
                           for the control panel applications.
   CSIDL_DESKTOP           Windows desktop -- virtual folder at the root of
                           the name space.
   CSIDL_DESKTOPDIRECTORY  File system directory used to physically store
                           file objects on the desktop (not to be confused
                           with the desktop folder itself).
   CSIDL_DRIVES            My Computer -- virtual folder containing
                           everything on the local computer: storage devices,
                           printers, and Control Panel. The folder may also
                           contain mapped network drives.
   CSIDL_FONTS             Virtual folder containing fonts.
   CSIDL_NETHOOD           File system directory containing objects that
                           appear in the network neighborhood.
   CSIDL_NETWORK           Network Neighborhood -- virtual folder representing
                           the top level of the network hierarchy.
   CSIDL_PERSONAL          File system directory that serves as a common
                           respository for documents.
   CSIDL_PRINTERS          Printers folder -- virtual folder containing
                           installed printers.
   CSIDL_PROGRAMS          File system directory that contains the user's
                           program groups (which are also file system
                           directories).
   CSIDL_RECENT            File system directory that contains the user's
                           most recently used documents.
   CSIDL_SENDTO            File system directory that contains Send To menu
                           items.
   CSIDL_STARTMENU         File system directory containing Start menu items.
   CSIDL_STARTUP           File system directory that corresponds to the
                           user's Startup program group.
   CSIDL_TEMPLATES         File system directory that serves as a common
                           repository for document templates.
   */

   if ( SHGetSpecialFolderLocation( FOXWINDOW()  ,
                                    _parni(1)    ,
                                    &pidlFolder
                                  ) == NOERROR )
      {
         if ( SHGetPathFromIDList( pidlFolder,(LPSTR) pszPath ) == TRUE )
            _retc( pszPath );
		 else
            _retc( "" );
      }
   else
      _retc( "" );

   FOCUSFNCRETURN();
}
 
HRESULT CreateLink( LPCSTR lpszPathObj,LPSTR lpszPathLink,LPSTR lpszDesc ) 
/*----------------------------------------------------------------------*/
{ 
   // The CreateLink function in the following example creates a shortcut.
   // The parameters include a pointer to the name of the file to link to,
   // a pointer to the name of the shortcut that you are creating, and a
   // pointer to the description of the link. The description consists of
   // the string, "Shortcut to filename," where filename is the name of the
   // file to link to. 
   // Because CreateLink calls the CoCreateInstance function, it is assumed
   // that the CoInitialize function has already been called. CreateLink
   // uses the IPersistFile interface to save the shortcut and the
   // IShellLink interface to store the filename and description.
   // CreateLink - uses the shell's IShellLink and IPersistFile interfaces 
   // to create and store a shortcut to the specified object. 
   // Returns the result of calling the member functions of the interfaces. 
   // lpszPathObj - address of a buffer containing the path of the object 
   // lpszPathLink - address of a buffer containing the path where the 
   // shell link is to be stored 
   // lpszDesc - address of a buffer containing the description of the 
   // shell link 

   HRESULT hres; 
   IShellLink* psl; 
 
   // Get a pointer to the IShellLink interface. 
   hres = CoCreateInstance( &CLSID_ShellLink,
                            NULL, 
                            CLSCTX_INPROC_SERVER,
                            &IID_IShellLink,
                            &psl); 

   if ( SUCCEEDED( hres ) )
      { 
         IPersistFile* ppf; 
 
         // Set the path to the shortcut target, and add the 
         // description. 
         psl->lpVtbl->SetPath(        psl,lpszPathObj ); 
         psl->lpVtbl->SetDescription( psl,lpszDesc    ); 
 
         // Query IShellLink for the IPersistFile interface for saving the 
         // shortcut in persistent storage. 
         hres = psl->lpVtbl->QueryInterface( psl,&IID_IPersistFile,&ppf ); 
 
         if ( SUCCEEDED( hres ) )
            { 
               WORD wsz[MAX_PATH]; 
 
               // Ensure that the string is ANSI. 
               MultiByteToWideChar( CP_ACP,0,lpszPathLink,-1,wsz,MAX_PATH );
 
               // Save the link by calling IPersistFile::Save. 
               hres = ppf->lpVtbl->Save( ppf,wsz,TRUE );
               ppf->lpVtbl->Release(ppf); 
            } 

          psl->lpVtbl->Release(psl); 
      } 

   return hres; 
} 

FOCUSFNC FW_FIL_CreateLink( XBASE_PARAMETERS )
/*------------------------------------------*/ 
{
   _initparc(1);_initparc(2);_initparc(3);

   CreateLink( _parc(1),_parc(2),_parc(3) );

   _retl( 1);

   _deinitparc(1);_deinitparc(2);_deinitparc(3);

   FOCUSFNCRETURN();
}

FOCUSFNC FW_FIL_ReadAllSections( XBASE_PARAMETERS )
/*-----------------------------------------------*/ 
{
   MHANDLE mh_out;        // Memory handle
   LPSTR   cAllSections;
   char    *lpszFile;
   UINT    wReturn;

   cAllSections = (LPSTR) _xgrab( (unsigned int) 1026,&mh_out );
   _MemFill( (void FAR *) cAllSections,0,1026 );

   _initparc(1);

   lpszFile = _parc(1);

   wReturn = GetPrivateProfileSections( (LPSTR) cAllSections,1024,(LPSTR) lpszFile );

   _deinitparc(1);

   if ( wReturn == 1 )
      _retc( cAllSections );
   else
      _retc( "" );

   _xfree( mh_out );

   FOCUSFNCRETURN();
}

UINT GetPrivateProfileSections( LPSTR lpOutBuf,UINT cbMax,LPSTR lpszFile )
/*----------------------------------------------------------------------*/
{
   MHANDLE        mh_out;        // Memory handle
   int            hFile ;
   LPSTR          cLine;
   LPSTR          cStartOfLine;
   LPSTR          cEndOfLine;
   LPSTR          cEndOfBuf;
   
   if ( ( hFile = (int) _FOpen( (char FAR *) lpszFile,FO_READONLY ) ) == -1 )
      return 0 ;  // Couldn't open INI file lpszFile

   // Allocate memory and get a pointer on a string of char
   cStartOfLine = cLine = (LPSTR) _xgrab( (unsigned int) 513,&mh_out );

   // Fill it with zeroes
   _MemFill( (void FAR *) cLine,0,(unsigned int) 513 );

   cEndOfBuf = lpOutBuf + cbMax - 1;

   __try {

   while ( ! _FEOF( hFile ) )
      {
         cEndOfLine = cStartOfLine + _FGets( hFile,cStartOfLine,512 );
		 cLine      = cStartOfLine;

         // Skip all leading spaces
		 while ( ( *cLine == ' ' ) & ( cLine <= cEndOfLine ) )
		    cLine++;

		 // If line doesn't start with a section, get to next line !
         if ( *cLine != '[' )
            continue;

         // Go to next character (first one after the '[')
         cLine++;

		 // While we are in the section header, recopy the section name in the output buffer
		 while ( ( *cLine != ']' ) & ( cLine <= cEndOfLine ) & ( lpOutBuf < cEndOfBuf ) )
            *lpOutBuf++ = *cLine++;

		 // Separate the section name form the other by a ',' if we do not exceed the buffer capacity
		 if ( lpOutBuf >= cEndOfBuf )
		   {
		      *cEndOfBuf = '\0'; return TRUE;
		   }
		 else
		   {
              *lpOutBuf++ = ',';
           }
      }

   // Let's get rid of the last ','
   if ( *( lpOutBuf - 1 ) == ',' )
      *( lpOutBuf - 1) = '\0';
   else
      *lpOutBuf = '\0';
   }

   __finally {
   // Close the file and free up the allocated memory
   _FClose( hFile ); _xfree( mh_out );
   }

   return TRUE;
}

FOCUSFNC FW_FIL_FileType( XBASE_PARAMETERS )
/*----------------------------------------*/
{
   HANDLE hFile;
   DWORD nFileType;

   _initparc(1);

   hFile = CreateFile( _parc(1)                          ,
                       GENERIC_READ                      ,
                       FILE_SHARE_READ | FILE_SHARE_WRITE,
                       NULL                              ,
                       OPEN_EXISTING                     ,
                       FILE_ATTRIBUTE_NORMAL             ,
                       NULL
                     );

   _deinitparc( 1 );

   if ( hFile != INVALID_HANDLE_VALUE )
     {
        nFileType = GetFileType( hFile );
        CloseHandle( hFile );
        _retni( (int) nFileType );

        // FILE_TYPE_UNKNOWN  The type of the specified file is unknown.
        // FILE_TYPE_DISK     The specified file is a disk file.
        // FILE_TYPE_CHAR     The specified file is a character file, typically an LPT device or a console.
        // FILE_TYPE_PIPE     The specified file is either a named or anonymous pipe.
        // FILE_TYPE_REMOTE

     }
   else
     {
	     _retni( -1 );
     }

   FOCUSFNCRETURN();
}
